package com.aurellem.capture;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.Callable;

import com.aurellem.capture.audio.MultiListener;
import com.aurellem.capture.audio.WaveFileWriter;
import com.aurellem.capture.video.AVIVideoRecorder;
import com.aurellem.capture.video.AbstractVideoRecorder;
import com.aurellem.capture.video.FileVideoRecorder;
import com.aurellem.capture.video.XuggleVideoRecorder;
import com.jme3.app.Application;
import com.jme3.audio.AudioRenderer;
import com.jme3.renderer.ViewPort;
import com.jme3.scene.Spatial;
import com.jme3.system.AppSettings;

public class Capture {

	public static void captureVideo(final Application app, final File file) throws IOException{

		final AbstractVideoRecorder videoRecorder;
		// The XuggleVideoRecorder is better than the AVIVideoRecorder in every way
		// except for ease of installation.  The excellent work by Werner Randelshofer
		// is used as a fallback option.  Please visit http://www.xuggle.com/ to learn
		// how to set up the XuggleVideoRecorder.

		if (file.getCanonicalPath().endsWith(".avi")){
			videoRecorder = new AVIVideoRecorder(file);}
		else if (file.isDirectory()){
			videoRecorder = new FileVideoRecorder(file);}
		else { videoRecorder = new XuggleVideoRecorder(file);}

		Callable<Object> thunk = new Callable<Object>(){
			public Object call(){

				ViewPort viewPort = 
						app.getRenderManager()
						.createPostView("aurellem record", app.getCamera());

				viewPort.setClearFlags(false, false, false);

				// get GUI node stuff
				for (Spatial s : app.getGuiViewPort().getScenes()){
					viewPort.attachScene(s);
				}

				app.getStateManager().attach(videoRecorder);
				viewPort.addProcessor(videoRecorder);
				return null;
			}
		};
		app.enqueue(thunk);
	}

	
	public static void captureAudio(final Application app, final File file) throws IOException{
		AppSettings settings = new AppSettings(true);
		settings.setAudioRenderer("Send");
		app.setSettings(settings);
		final WaveFileWriter writer = new WaveFileWriter(file);

		Callable<Object> thunk = new Callable<Object>(){
			public Object call(){
				AudioRenderer ar = app.getAudioRenderer();
				if (ar instanceof MultiListener){
					MultiListener ml = (MultiListener)ar;
					ml.registerSoundProcessor(writer);
				}
				return null;
			}
		};
		
		app.enqueue(thunk);
	}
	
}
